/* Log data from the TCP socket which is open on fd 0.
 * Warren Toomey, wkt@cs.adfa.edu.au
 *
 * $Date: 1999/05/26 01:27:51 $
 */

#include "header.h"
char *inet_ntoa();

struct sockaddr_in client;
struct sockaddr_in server;
int fd;
char buf[BUFSIZ];		/* Buffer of incoming data */
int bytecount = 0;		/* Bytes in the buffer */
int bytesleft = BUFSIZE;	/* Bytes left */
char *bptr = buf;		/* Pointer to insertion point */

void main(int argc, char *argv[])
{
  struct hostent *hp;
  struct servent *sp;
  char *dotip, *name, *servname;
  int len;
  struct timeval waittime;

  fd = 0; openlog("tcpsuck", LOG_PID, LOG_PACKETS);

				/* Set default timeout to 60 seconds,
				 * or to 1st arg if between 0 and 600.
				 */
  waittime.tv_sec = 60; waittime.tv_usec = 0;
  if ((argc > 1) && ((len = atoi(argv[1])) > 0) && (len < 600))
    waittime.tv_sec = len;

  if (setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &waittime, sizeof(waittime)) < 0)
    mop_warn("setsockopt: %m");

  len = sizeof(server);
  if (getsockname(fd, (struct sockaddr *) & server, &len) < 0)
    mop_warn("getsockname: %m");

  len = sizeof(client);
  if (getpeername(fd, (struct sockaddr *) & client, &len) < 0) {
    len = sizeof(client);
    if (recvfrom(fd, buf, sizeof(buf), MSG_PEEK, (struct sockaddr *) &client,
								   &len) < 0) {
      mop_warn("can't get client address: %m");
    }
  }
  hp = gethostbyaddr((char *) &(client.sin_addr), sizeof(client.sin_addr),
								    AF_INET);
  if (hp && hp->h_name) name = hp->h_name;
  else name = "UNKNOWN";

  sp = getservbyport(server.sin_port, "tcp");
  if (sp && sp->s_name) servname = sp->s_name;
  else servname = "unknown";

  dotip = inet_ntoa(client.sin_addr);
  if (dotip == NULL) dotip = "UNKNOWN";

					/* Loop reading data until the
					 * buffer is full, or the
					 * timeout goes off
					 */
  while (1) {
    len = read(fd, bptr, bytesleft);
    if (len < 1) break;
    bptr += len; bytecount += len; bytesleft -= len;
    if (bytesleft < 1) break;
  }

					/* And now log it */
  if (bytecount) {
    syslog(LOG_NOTICE, "Data from %s (%s) port %d to %s (port %d)",
	 name, dotip, ntohs(client.sin_port), servname,
	 ntohs(server.sin_port));
    dump_packet(buf, bytecount);
  }
  exit(0);
}
